@@ -258,6 +258,7 @@ int _cl_delete_corpus(SEXP corpus, SEXP registry){
   
   Corpus * c;
   static char *canonical_name = NULL;
+  int retval;
   
   char* registry_dir = strdup(Rcpp::as<std::string>(registry).c_str());
   char* registry_name  = strdup(Rcpp::as<std::string>(corpus).c_str());
@@ -270,18 +271,47 @@ int _cl_delete_corpus(SEXP corpus, SEXP registry){
     Rprintf("cl_new_corpus: <%s> is not a valid corpus name\n", registry_name);
   }
   
-  Rprintf("Corpus to delete (ID): %s\n", registry_name);
   c = find_corpus(registry_dir, canonical_name); 
-  Rprintf("Corpus name: %s\n", c->name);
-  Rprintf("Number of loads before reset: %d\n", c->nr_of_loads);
-  c->nr_of_loads = 1;
-  Rprintf("Number of loads resetted: %d\n", c->nr_of_loads);
-  cl_delete_corpus(c);
+  if (c){
+    c->nr_of_loads = 1;
+    cl_delete_corpus(c);
+    retval = 1;
+  } else {
+    retval = 0;
+  }
   
-  return( 0 );
+  return( retval );
 }
 
 
+// [[Rcpp::export(name=".corpus_is_loaded")]]
+int _corpus_is_loaded(SEXP corpus, SEXP registry){
+  
+  Corpus * c;
+  static char *canonical_name = NULL;
+  int retval;
+  
+  char* registry_dir = strdup(Rcpp::as<std::string>(registry).c_str());
+  char* registry_name  = strdup(Rcpp::as<std::string>(corpus).c_str());
+  
+  /* code copied from cl_new_corpus in corpus.c */
+  cl_free(canonical_name);
+  canonical_name = cl_strdup(registry_name);
+  cl_id_tolower(canonical_name);
+  if (!cl_id_validate(canonical_name)) {
+    Rprintf("cl_new_corpus: <%s> is not a valid corpus name\n", registry_name);
+  }
+  
+  c = find_corpus(registry_dir, canonical_name); 
+  if (c){
+    retval = 1;
+  } else {
+    retval = 0;
+  }
+  
+  return( retval );
+}
+
 
 // [[Rcpp::export(name=".cl_charset_name")]]
 Rcpp::StringVector _cl_charset_name(SEXP corpus, SEXP registry){
@@ -298,8 +328,6 @@ Rcpp::StringVector _cl_charset_name(SEXP corpus, SEXP registry){
 }
 
 
-
-
 // [[Rcpp::export(name=".init_cqp")]]
 void init_cqp() {
 	int		ac = 1;
@@ -322,7 +350,8 @@ void init_cqp() {
 // [[Rcpp::export(name=".cqp_get_registry")]]
 Rcpp::StringVector cqp_get_registry(){
   Rcpp::StringVector result(1);
-  result(0) = cl_standard_registry();
+  /* result(0) = cl_standard_registry(); */
+  result(0) = registry;
   return result;
 }
 
@@ -334,9 +363,8 @@ int cqp_get_status(){
 
 // [[Rcpp::export(name=".cqp_set_registry")]]
 SEXP cqp_set_registry(SEXP registry_dir){
-  char * registry_new;
-  registry_new = (char*)CHAR(STRING_ELT(registry_dir,0));
-  registry = cl_strdup(registry_new);
+  
+  registry = strdup(Rcpp::as<std::string>(registry_dir).c_str());
   int		ac = 1;
   char *		av[1];
   av[0] = (char *)"RcppCWB";
@@ -574,8 +602,13 @@ Rcpp::StringVector _corpus_data_dir(SEXP corpus, SEXP registry){
   char* registry_dir = strdup(Rcpp::as<std::string>(registry).c_str());
   
   c = cl_new_corpus(registry_dir, corpus_id);
-
-  result(0) = c->path;
+  
+  if (c){
+    result(0) = c->path;
+  } else {
+    result(0) = NA_STRING;
+  }
+  
   return( result );
 
 }
